from matplotlib import pyplot, patches
from numpy import (cos, tan, deg2rad, sin, pi, sqrt, rad2deg, deg2rad, arctan)

pyplot.rcParams['svg.fonttype'] = 'none' # Export font as text in .svg

class Drawer():
    def __init__(self, projection):
        """
        """
        self.projection = projection
    
    def draw(self, ax, degree_steps, center_length = 0.1, external_radius = 11.0):
        self.draw_circles(ax)
        self.draw_hauteurs(ax, degree_steps)
        self.draw_azimuts(ax, degree_steps)
        ax.add_patch(pyplot.Circle((0, 0), external_radius, color='red', fill=False, linewidth=1))
        ax.plot([-center_length, center_length], [0, 0])
        ax.plot([0, 0], [-center_length, center_length])
        #texts = ["XII","XI","X","IX","VIII","VII","VI","V","IV","III","II","I"] * 2
        texts = ["12","11","10","9","8","7","6","5","4","3","2","1"] * 2
        self.draw_graduated_circle(ax, self.projection.projection_radius, division=24, sub_division=3, length=0.3, sub_length=0.1, texts=texts, phase=-90)                   
        ax.axis('scaled')

    def draw_circles(self, ax):
        ax.add_patch(pyplot.Circle((0, 0), self.projection.project_latitude(0, -self.projection.tropical_inclinaison), color='red', fill=False, linewidth=1))
        ax.add_patch(pyplot.Circle((0, 0), self.projection.project_latitude(0, self.projection.tropical_inclinaison), color='green', fill=False, linewidth=1))
    
    def draw_hauteurs(self, ax, degree_steps):
        ranges = [90.0*float(float(i)/float(degree_steps)) for i in range(0, degree_steps+1)] 
        for hauteur in ranges+[96, 102, 108]:
            h, r = self.projection.project_latitude(hauteur), self.projection.project_latitude(hauteur, sign=-1)
            theta = 180 if abs(h) + abs(r) < self.projection.projection_radius else self.projection.angle_arc(r, self.projection.projection_radius, h)
            ax.add_patch(patches.Arc((0, h), 2*r, 2*r, theta1=-theta, theta2=theta, angle=-90, color="blue", alpha=1.0))
    
    def draw_azimut(self, ax, angle, horizon_center, height):

        y = horizon_center - self.projection.project_latitude(90)

        azimuth_position = tan(deg2rad(angle))*height
        azimuth_radius = height / cos(deg2rad(angle))

        horizon_distance = sqrt(azimuth_position**2 + y**2)
        capricorn_distance = sqrt(azimuth_position**2 + horizon_center**2)

        theta_cap = 180.0
        if capricorn_distance + azimuth_radius > self.projection.capricorn_radius():
            theta_cap = self.projection.angle_arc(azimuth_radius, self.projection.capricorn_radius(), capricorn_distance)
        theta_hor = self.projection.angle_arc(azimuth_radius, self.projection.horizon_radius(), horizon_distance)
        
        angle_hor = rad2deg(arctan(azimuth_position/y))
        angle_cap = rad2deg(arctan(azimuth_position/horizon_center))

        theta1 = max(-theta_cap - angle_cap, -theta_hor - angle_hor)
        theta2 = min(theta_cap - angle_cap, theta_hor - angle_hor)

        ax.add_patch(patches.Arc((azimuth_position, horizon_center), 2*azimuth_radius, 2*azimuth_radius, angle=90, theta1=theta1, theta2=theta2, alpha=1.0))

    def draw_azimuts(self, ax, degree_steps):
        horizon_center = self.projection.project_latitude(90, self.projection.local_latitude + 90)
        height = self.projection.project_latitude(90, self.projection.local_latitude + 90, sign=-1)
        ranges = [90.0*float(float(i)/float(degree_steps)) for i in range(0, degree_steps)] + [-90.0*float(float(i)/float(degree_steps)) for i in range(1, degree_steps)]
        north = self.projection.project_latitude(90) - self.projection.horizon_radius()
        south = min(self.projection.capricorn_radius(), self.projection.project_latitude(90) + self.projection.horizon_radius())
        ax.plot((0,0),(north, south), color="black", alpha=1.0, linewidth=1)
        for a in ranges:
            self.draw_azimut(ax, a, horizon_center, height)

    def draw_graduated_circle(self, ax, radius, division, sub_division, length, sub_length, center_x=0, center_y=0, texts=[], phase=0):
        for i in range(division*sub_division):
            angle = 2*pi*i/(division*sub_division) + deg2rad(phase)
            graduation_length = length if i % sub_division == 0 else sub_length
            X = (radius * cos(angle) + center_x, (radius + graduation_length) * cos(angle) + center_x)
            Y = (radius * sin(angle) + center_y, (radius + graduation_length) * sin(angle) + center_y)
            ax.plot(X, Y, color='black')
            if i % sub_division == 0 and i//sub_division < len(texts):
                x, y = (radius + length * 2) * (cos(angle) + center_x), (radius + length * 2) * (sin(angle) + center_y)
                ax.text(x, y, texts[i//sub_division], rotation=rad2deg(angle - pi/2.0),horizontalalignment='center', verticalalignment='center', fontsize=16)
